package default_builtins

import (
	"fmt"
	"git.swzry.com/ProjectNagae/saki-v0/engine"
	"github.com/dop251/goja"
	"os"
	"os/exec"
	"time"
)

type GlobalObjects struct {
	mctx *engine.ModuleContext
}

func (g *GlobalObjects) Enable(mctx *engine.ModuleContext) engine.RegisterFunc {
	g.mctx = mctx
	return g.register
}

func (g *GlobalObjects) register(s engine.ModuleSetFunc) {
	s("print", g.fPrint)
	s("println", g.fPrintln)
	s("printf", g.fPrintf)
	s("execute", g.fExecute)
	s("delay", g.fDelayMilliSec)
}

func (g *GlobalObjects) iPrintArgPrepare(call goja.FunctionCall) []any {
	printItems := make([]any, len(call.Arguments))
	for i, arg := range call.Arguments {
		printItems[i] = arg.String()
	}
	return printItems
}

func (g *GlobalObjects) iWarn(err error) {
	if g.mctx != nil {
		g.mctx.RuntimeWarning(err)
	}
}

func (g *GlobalObjects) fPrint(call goja.FunctionCall) goja.Value {
	fmt.Print(g.iPrintArgPrepare(call)...)
	return goja.Undefined()
}

func (g *GlobalObjects) fPrintln(call goja.FunctionCall) goja.Value {
	fmt.Println(g.iPrintArgPrepare(call)...)
	return goja.Undefined()
}

func (g *GlobalObjects) fPrintf(call goja.FunctionCall) goja.Value {
	l := len(call.Arguments)
	if l < 1 {
		panic(g.mctx.NewValue(fmt.Errorf("printf: not enough arguments")))
	}
	formatRaw := call.Arguments[0].Export()
	format, ok := formatRaw.(string)
	if !ok {
		panic(g.mctx.NewValue(fmt.Errorf("printf: argument[0] is not string")))
	}
	args := make([]any, l-1)
	for i := 1; i < l; i++ {
		args[i-1] = call.Arguments[i].Export()
	}
	fmt.Printf(format, args...)
	return goja.Undefined()
}

func (g *GlobalObjects) fExecute(call goja.FunctionCall) goja.Value {
	l := len(call.Arguments)
	if l < 1 {
		panic(g.mctx.NewValue(fmt.Errorf("execute: not enough arguments")))
	}
	args := make([]string, l)
	for i := 0; i < l; i++ {
		v, ok := call.Arguments[i].ToString().Export().(string)
		if !ok {
			panic(g.mctx.NewValue(fmt.Errorf("execute: argument[%d] is not string", i)))
		}
		args[i] = v
	}
	bin := args[0]
	args = args[1:]
	cmd := exec.Command(bin, args...)
	cmd.Stdin = os.Stdin
	cmd.Stdout = os.Stdout
	cmd.Stderr = os.Stderr
	err := cmd.Run()
	if err != nil {
		panic(g.mctx.NewValue(fmt.Errorf("execute: error in executing command '%s': %w", bin, err)))
	}
	res := cmd.ProcessState.ExitCode()
	return g.mctx.NewValue(res)
}

func (g *GlobalObjects) fDelayMilliSec(call goja.FunctionCall) goja.Value {
	if len(call.Arguments) != 1 {
		panic(g.mctx.NewValue(fmt.Errorf("delay: invalid arguments")))
	}
	ms := call.Arguments[0].ToInteger()
	time.Sleep(time.Duration(ms) * time.Millisecond)
	return goja.Undefined()
}
